""" Tests for the acl dataset
"""
from tempfile import NamedTemporaryFile
import unittest

from rbldnsd import ZoneFile, Rbldnsd, QueryRefused

__all__ = [
    'TestAclDataset',
    ]

def daemon(acl, addr='localhost'):
    """ Create an Rbldnsd instance with given ACL
    """
    acl_zone = NamedTemporaryFile()
    acl_zone.writelines("%s\n" % line for line in acl)
    acl_zone.flush()

    dnsd = Rbldnsd(daemon_addr=addr)
    dnsd.add_dataset('acl', acl_zone)
    dnsd.add_dataset('generic', ZoneFile(['test TXT "Success"']))
    return dnsd

class TestAclDataset(unittest.TestCase):
    def test_refuse_ipv4(self):
        with daemon(acl=["127.0.0.1 :refuse"],
                    addr='127.0.0.1') as dnsd:
            self.assertRaises(QueryRefused, dnsd.query, 'test.example.com')

    def test_pass_ipv4(self):
        with daemon(acl=[ "0.0.0.0/0 :refuse",
                          "127.0.0.1 :pass" ],
                    addr='127.0.0.1') as dnsd:
            self.assertEqual(dnsd.query('test.example.com'), 'Success')

    def test_refuse_ipv6(self):
        with daemon(acl=["::1 :refuse"],
                    addr='::1') as dnsd:
            self.assertRaises(QueryRefused, dnsd.query, 'test.example.com')

    def test_pass_ipv6(self):
        with daemon(acl=[ "0/0 :refuse",
                          "0::1 :pass" ],
                    addr='::1') as dnsd:
            self.assertEqual(dnsd.query('test.example.com'), 'Success')

if __name__ == '__main__':
    unittest.main()



""" Tests for btrie.c
"""

import os
import re
from subprocess import Popen, PIPE
from tempfile import TemporaryFile, NamedTemporaryFile
import unittest

from rbldnsd import Rbldnsd, ZoneFile

__all__ = [
    'Test_coalesce_lc_node',
    'Test_shorten_lc_node',
    'Test_convert_lc_node_1',
    'Test_convert_lc_node',
    'Test_insert_lc_node',
    'Test_init_tbm_node',
    'Test_add_to_trie',
    'Test_search_trie',
    ]

def deduce_pointer_size(makefile='./Makefile'):
    """ Deduce the pointer size (in the current compilation environment)
    """
    with file(makefile) as f:
        make_vars = dict(
            m.groups()
            for m in (re.match(r'\s*(\w+)\s*=\s*(.*?)\s*\Z', line)
                      for line in f)
            if m is not None)
    cc = make_vars['CC']
    cflags = make_vars['CFLAGS']

    test_c = NamedTemporaryFile(suffix=".c")
    test_c.write(r'''
#include <stdio.h>
#ifndef __SIZEOF_POINTER__
# define __SIZEOF_POINTER__ sizeof(void *)
#endif
int main () {
  printf("%u\n", (unsigned)__SIZEOF_POINTER__);
  return 0;
}
''')
    test_c.flush()
    src = test_c.name

    try:
        proc = Popen("%(cc)s %(cflags)s -o %(src)s.bin %(src)s && %(src)s.bin"
                     % locals(),
                     shell=True, stdout=PIPE)
        output = proc.stdout.read()
        if proc.wait() != 0:
            raise RuntimeError("test prog exited with code %d"
                               % proc.returncode)
        return int(output)
    finally:
        try:
            os.unlink(src + '.bin')
        except:
            pass

try:
    sizeof_pointer = deduce_pointer_size()
except Exception:
    print "Can not deduce size of pointer. Assuming pointer size of 8."
    sizeof_pointer = 8

if sizeof_pointer == 8:
    STRIDE = 5
    LC_BYTES_PER_NODE = 7
elif sizeof_pointer == 4:
    STRIDE = 4
    LC_BYTES_PER_NODE = 3
else:
    raise RuntimeError("Unsupported pointer size (%d)" % sizeof_pointer)

def pad_prefix(prefix, plen):
    """Pad prefix on the right with zeros to a full 128 bits
    """
    if not isinstance(prefix, (int, long)):
        raise TypeError("prefix must be an integer")
    if not 0 <= int(plen) <= 128:
        raise ValueError("plen out of range")
    if not 0 <= prefix < (1 << plen):
        raise ValueError("prefix out of range")
    return prefix << (128 - plen)

class BTrie(object):
    """ A class to construct and perform lookups on a btrie.

    Since we do not have python bindings for btrie, we do this in a
    roundabout way by running rbldnsd with a single ip6trie dataset,
    and then querying the rbldnsd to perform the lookup.

    """
    def __init__(self, prefixes, **kwargs):
        self.rbldnsd = Rbldnsd(**kwargs)
        zonedata = (self._zone_entry(*prefix) for prefix in prefixes)
        self.rbldnsd.add_dataset('ip6trie', ZoneFile(zonedata))

    def __enter__(self):
        self.rbldnsd.__enter__()
        return self

    def __exit__(self, exc_type, exc_value, exc_tb):
        return self.rbldnsd.__exit__(exc_type, exc_value, exc_tb)

    def lookup(self, prefix, plen):
        prefix = pad_prefix(prefix, plen)
        nibbles = '.'.join("%x" % ((prefix >> n) & 0x0f)
                           for n in range(0, 128, 4))
        return self.rbldnsd.query(nibbles + '.example.com')

    def _zone_entry(self, prefix, plen, data):
        prefix = pad_prefix(prefix, plen)
        ip6addr = ':'.join("%x" % ((prefix >> n) & 0xffff)
                           for n in range(112, -16, -16))
        return "%s/%u :1:%s" % (ip6addr, plen, data)

class CaptureOutput(object):
    def __init__(self):
        self._file = TemporaryFile()

    def fileno(self):
        return self._file.fileno()

    def __contains__(self, substr):
        return substr in str(self)

    def __str__(self):
        self._file.seek(0, 0)
        return self._file.read()

class Test_coalesce_lc_node(unittest.TestCase):
    def test_merge(self):
        # test coverage of coalesce_lc_node
        prefixes = [
            # this prefix is too long for a single LC node
            # but after we stick the TBM node at 0/0 it should
            # just fit into a single LC extension path
            (0, 8 * (LC_BYTES_PER_NODE + 1), "term"),
            # Add a TBM node to shorten the above LC node
            (0, (8 - STRIDE), "root"),
            ]
        with BTrie(prefixes) as btrie:
            self.assertEqual(btrie.lookup(0, 8), "term")
            self.assertEqual(btrie.lookup(1, 8), "root")

    def test_steal_bits(self):
        # test coverage of coalesce_lc_node
        prefixes = [
            # This prefix is too long for a single LC node.  After we
            # stick the TBM node at 0/0 it should still be too long
            # for a single LC node, but the upper LC node should steal
            # bits from the terminal LC node.
            (0, 8 * (LC_BYTES_PER_NODE + 1) + 1, "term"),
            (0, (8 - STRIDE), "root"),
            ]
        with BTrie(prefixes) as btrie:
            self.assertEqual(btrie.lookup(0, 0), "term")
            self.assertEqual(btrie.lookup(1, 8), "root")

class Test_shorten_lc_node(unittest.TestCase):
    def test_steal_child(self):
        # test coverage of coalesce_lc_node
        prefixes = [
            # this prefix is too long for a single LC node
            # but after we stick the TBM node at 0/0 it should
            # just fit into a single LC extension path
            (0, 9, "tbm root"),
            (0, 10, "term"),
            # Add a TBM node to shorten the above LC node
            (0, 9 - STRIDE, "root"),
            ]
        with BTrie(prefixes) as btrie:
            self.assertEqual(btrie.lookup(0, 8), "term")
            self.assertEqual(btrie.lookup(1, 8), "root")

class Test_convert_lc_node_1(unittest.TestCase):
    def test_left_child(self):
        # test coverage of coalesce_lc_node
        prefixes = [
            # create TBM node at depth 1
            (0, 2, "term"),
            (0, 1, "tbm node"),
            # promote to depth 0
            (0, 0, "root"),
            ]
        with BTrie(prefixes) as btrie:
            self.assertEqual(btrie.lookup(0, 0), "term")
            self.assertEqual(btrie.lookup(1, 2), "tbm node")
            self.assertEqual(btrie.lookup(1, 1), "root")

    def test_right_child(self):
        # test coverage of coalesce_lc_node
        prefixes = [
            (3, 2, "term"),
            (1, 1, "tbm node"),
            (0, 0, "root"),
            ]
        with BTrie(prefixes) as btrie:
            self.assertEqual(btrie.lookup(3, 2), "term")
            self.assertEqual(btrie.lookup(2, 2), "tbm node")
            self.assertEqual(btrie.lookup(0, 1), "root")

class Test_convert_lc_node(unittest.TestCase):
    def test_left_child(self):
        # test coverage of coalesce_lc_node
        prefixes = [
            # create TBM node at depth STRIDE - 1
            (0, STRIDE, "term"),
            (0, STRIDE - 1, "tbm node"),
            # promote to depth 0
            (0, 0, "root"),
            ]
        with BTrie(prefixes) as btrie:
            self.assertEqual(btrie.lookup(0, STRIDE), "term")
            self.assertEqual(btrie.lookup(1, STRIDE), "tbm node")
            self.assertEqual(btrie.lookup(1, 1), "root")

class Test_insert_lc_node(unittest.TestCase):
    def test_insert_lc_len_1(self):
        prefixes = [
            # create TBM node at depth 1 with TBM extending path
            (0, STRIDE + 2, "term"),
            (0, STRIDE + 1, "tbm ext path"),
            (0, 1, "tbm node"),
            # promote to depth 0
            (0, 0, "root"),
            ]
        with BTrie(prefixes) as btrie:
            self.assertEqual(btrie.lookup(0, 0), "term")
            self.assertEqual(btrie.lookup(1, STRIDE + 2), "tbm ext path")
            self.assertEqual(btrie.lookup(1, 2), "tbm node")
            self.assertEqual(btrie.lookup(1, 1), "root")

    def test_extend_lc_tail_optimization(self):
        prefixes = [
            # create TBM node at depth 1 with LC extending path
            (1, STRIDE + 2, "term"),
            (0, 1, "tbm node"),
            # promote to depth 0
            (0, 0, "root"),
            ]
        with BTrie(prefixes) as btrie:
            self.assertEqual(btrie.lookup(1, STRIDE + 2), "term")
            self.assertEqual(btrie.lookup(0, 0), "tbm node")
            self.assertEqual(btrie.lookup(1, 1), "root")

    def test_coalesce_lc_tail(self):
        prefixes = [
            # create TBM node with LC extending path which starts
            # at a byte boundary.
            (0, 10, "term"),
            (0, 8 - STRIDE, "tbm node"),
            # promote one level
            (0, 7 - STRIDE, "promoted"),
            ]
        with BTrie(prefixes) as btrie:
            self.assertEqual(btrie.lookup(0, 0), "term")
            self.assertEqual(btrie.lookup(1, 9 - STRIDE), "tbm node")
            self.assertEqual(btrie.lookup(1, 8 - STRIDE), "promoted")

class Test_init_tbm_node(unittest.TestCase):
    def test_short_lc_children(self):
        # this exercises the convert_lc_node calls in init_tbm_node()
        prefixes = [
            # create TBM node at depth 1, with two LC extending paths
            # from a deep internal node
            (0, 1, "tbm"),
            (0, STRIDE + 2, "term0"),
            (2, STRIDE + 2, "term1"),
            # promote one level
            (0, 0, "root"),
            ]
        with BTrie(prefixes) as btrie:
            self.assertEqual(btrie.lookup(0, 0), "term0")
            self.assertEqual(btrie.lookup(1, STRIDE + 1), "term1")
            self.assertEqual(btrie.lookup(1, 2), "tbm")
            self.assertEqual(btrie.lookup(1, 1), "root")

    def test_long_lc_children(self):
        # this exercises the shorten_lc_node calls in init_tbm_node()
        prefixes = [
            # create TBM node at depth 1, with two LC extending paths
            # from a deep internal node
            (0, 1, "tbm"),
            (0, STRIDE + 9, "term0"),
            (0x100, STRIDE + 9, "term1"),
            # promote one level
            (0, 0, "root"),
            ]
        with BTrie(prefixes) as btrie:
            self.assertEqual(btrie.lookup(0, 0), "term0")
            self.assertEqual(btrie.lookup(1, STRIDE + 1), "term1")
            self.assertEqual(btrie.lookup(1, 2), "tbm")
            self.assertEqual(btrie.lookup(1, 1), "root")

    def test_set_internal_data_for_root_prefix(self):
        # this exercises the "set internal data for root prefix" code
        prefixes = [
            # create TBM node at depth 1, with internal prefix data
            # and an extending path on a deep internal node
            (0, 1, "tbm"),
            (0, STRIDE, "int data"),
            (0, STRIDE + 1, "ext path"),
            # promote one level
            (0, 0, "root"),
            ]
        with BTrie(prefixes) as btrie:
            self.assertEqual(btrie.lookup(0, 0), "ext path")
            self.assertEqual(btrie.lookup(1, STRIDE + 1), "int data")
            self.assertEqual(btrie.lookup(1, 2), "tbm")
            self.assertEqual(btrie.lookup(1, 1), "root")

    def test_set_right_ext_path(self):
        # this exercises the insert_lc_node(right_ext) call in init_tbm_node()
        # this also exercises next_pbyte with (pos + TBM_STRIDE) % 8 == 0
        prefixes = [
            # create TBM node at depth (9 - STRIDE) with a right TBM
            # extending path on a deep internal node
            (0, 9 - STRIDE, "tbm"),
            (1, 9, "ext path"),
            (2, 10, "term"),
            # promote one level to depth (8 - STRIDE)
            (0, 8 - STRIDE, "top"),
            ]
        with BTrie(prefixes) as btrie:
            self.assertEqual(btrie.lookup(0, 0), "tbm")
            self.assertEqual(btrie.lookup(2, 10), "term")
            self.assertEqual(btrie.lookup(3, 10), "ext path")
            self.assertEqual(btrie.lookup(1, 9 - STRIDE), "top")

class Test_add_to_trie(unittest.TestCase):
    def test_duplicate_terminal_lc(self):
        prefixes = [
            (0, 1, "term"),
            (0, 1, "term"),
            ]
        stderr = CaptureOutput()
        with BTrie(prefixes, stderr=stderr) as btrie:
            self.assertEqual(btrie.lookup(0, 0), "term")
        self.assertTrue("duplicated entry for" in stderr,
                        "No duplicated entry error message in stderr: %r"
                        % str(stderr))

    def test_duplicate_internal_data(self):
        prefixes = [
            (0, 0, "root"),
            (2, 3, "term"),
            (2, 3, "term"),
            ]
        stderr = CaptureOutput()
        with BTrie(prefixes, stderr=stderr) as btrie:
            self.assertEqual(btrie.lookup(4, 4), "term")
            self.assertEqual(btrie.lookup(0, 0), "root")
        self.assertTrue("duplicated entry for" in stderr,
                        "No duplicated entry error message in stderr: %r"
                        % str(stderr))

    def test_split_first_byte_of_lc_prefix(self):
        # this is for coverage of common_prefix()
        prefixes = [
            (0x1234, 16, "long"),
            (0x1000, 16, "splitter"),
            ]
        with BTrie(prefixes) as btrie:
            self.assertEqual(btrie.lookup(0x1234, 16), "long")
            self.assertEqual(btrie.lookup(0x1000, 16), "splitter")

    def test_split_last_byte_of_lc_prefix(self):
        # this is for coverage of common_prefix()
        prefixes = [
            (0x1234, 15, "long"),
            (0x1238, 15, "splitter"),
            ]
        with BTrie(prefixes) as btrie:
            self.assertEqual(btrie.lookup(0x1234, 15), "long")
            self.assertEqual(btrie.lookup(0x1238, 15), "splitter")

class Test_search_trie(unittest.TestCase):
    def test_tbm_root_data(self):
        # test access to root internal node in a TBM node
        prefixes = [(0, 127, "tbm root"),
                    (1, 128, "int data")]
        with BTrie(prefixes) as btrie:
            self.assertEqual(btrie.lookup(0, 0), "tbm root")

    def test_tbm_internal_data(self):
        # test access to each (non-root) internal node in a TBM node
        for plen in range(1, STRIDE):
            # TBM node
            prefixes = [(0, 128 - plen, "tbm root")]
            prefixes.extend((pfx, 128, "%u/%u" % (pfx, plen))
                            for pfx in range(1 << plen))
            with BTrie(prefixes) as btrie:
                for pfx in range(1 << plen):
                    self.assertEqual(btrie.lookup(pfx, 128),
                                     "%u/%u" % (pfx, plen))

    def test_tbm_extending_paths(self):
        # test access to each extended path of a TBM node
        prefixes = [(0,0,"root")] # make sure to create top-level TBM node
        prefixes.extend((pfx, STRIDE, str(pfx)) for pfx in range(1 << STRIDE))
        with BTrie(prefixes) as btrie:
            for pfx in range(1 << STRIDE):
                self.assertEqual(btrie.lookup(pfx, STRIDE), str(pfx))

    def test_no_match(self):
        prefixes = [
            (1, 2, "term"),
            ]
        with BTrie(prefixes) as btrie:
            self.assertEqual(btrie.lookup(0,0), None)

    def test_follow_lc(self):
        prefixes = [
            (0, 2 * STRIDE, "term"),
            ]
        with BTrie(prefixes) as btrie:
            self.assertEqual(btrie.lookup(0,0), "term")

    def test_parents_internal_data(self):
        prefixes = [
            (0, 0, "root"),
            (2, 2, "int data"),
            (0x200, 10, "term"),
            ]
        with BTrie(prefixes) as btrie:
            self.assertEqual(btrie.lookup(0x201, 10), "int data")

if __name__ == '__main__':
    unittest.main()



""" Basic ip6trie dataset Tests
"""
import unittest

from rbldnsd import Rbldnsd, ZoneFile

__all__ = [
    'TestIp6TrieDataset',
    ]

def ip6trie(zone_data):
    """ Run rbldnsd with an ip6trie dataset
    """
    dnsd = Rbldnsd()
    dnsd.add_dataset('ip6trie', ZoneFile(zone_data))
    return dnsd

def rfc3152(ip6addr, domain='example.com'):
    from socket import inet_pton, AF_INET6
    from struct import unpack

    bytes = unpack("16B", inet_pton(AF_INET6, ip6addr))
    nibbles = '.'.join("%x.%x" % (byte & 0xf, (byte >> 4) & 0xf)
                       for byte in reversed(bytes))
    return "%s.%s" % (nibbles, domain)

class TestIp6TrieDataset(unittest.TestCase):
    def test_exclusion(self):
        with ip6trie(["dead::/16 listed",
                      "!dead::beef"]) as dnsd:
            self.assertEqual(dnsd.query(rfc3152("dead::beef")), None)
            self.assertEqual(dnsd.query(rfc3152("dead::beee")), "listed")

if __name__ == '__main__':
    unittest.main()



""" (Very) basic ip4trie dataset tests
"""
import unittest

from rbldnsd import Rbldnsd, ZoneFile

__all__ = [
    'TestIp4TrieDataset',
    ]

def ip4trie(zone_data):
    """ Run rbldnsd with an ip6trie dataset
    """
    dnsd = Rbldnsd()
    dnsd.add_dataset('ip4trie', ZoneFile(zone_data))
    return dnsd

def reversed_ip(ip4addr, domain='example.com'):
    revip = '.'.join(reversed(ip4addr.split('.')))
    return "%s.%s" % (revip, domain)

class TestIp4TrieDataset(unittest.TestCase):
    def test_exclusion(self):
        with ip4trie(["1.2.3.0/24 listed",
                      "!1.2.3.4"]) as dnsd:
            self.assertEqual(dnsd.query(reversed_ip("1.2.3.4")), None)
            self.assertEqual(dnsd.query(reversed_ip("1.2.3.3")), "listed")
            self.assertEqual(dnsd.query(reversed_ip("1.2.3.5")), "listed")

    def test_wildcard_prefix(self):
        with ip4trie(["0/0 wild",
                      "127.0.0.1 localhost"]) as dnsd:
            self.assertEqual(dnsd.query(reversed_ip("127.0.0.1")), "localhost")
            self.assertEqual(dnsd.query(reversed_ip("0.0.0.0")), "wild")
            self.assertEqual(dnsd.query(reversed_ip("127.0.0.2")), "wild")

if __name__ == '__main__':
    unittest.main()



""" Run an rbldnsd and send it DNS queries.


"""
from itertools import count
import subprocess
from tempfile import NamedTemporaryFile, TemporaryFile
import time
import unittest

try:
    import DNS
except ImportError:
    raise RuntimeError("The pydns library is not installed")

DUMMY_ZONE_HEADER = """
$SOA 0 example.org. hostmaster.example.com. 0 1h 1h 2d 1h
$NS 1d ns0.example.org
"""

class ZoneFile(object):
    def __init__(self, lines=None, no_header=False):
        self._file = NamedTemporaryFile()
        if not no_header:
            self._file.write(DUMMY_ZONE_HEADER)
        if lines is not None:
            self.writelines(lines)
        self._file.flush()

    @property
    def name(self):
        return self._file.name

    def write(self, str):
        self._file.write(str)
        self._file.flush()

    def writelines(self, lines):
        self._file.writelines("%s\n" % line for line in lines)
        self._file.flush()

class DaemonError(Exception):
    """ Various errors having to do with the execution of the daemon.
    """

class QueryRefused(Exception):
    """ Query to rbldnsd was REFUSED.
    """


class Rbldnsd(object):
    def __init__(self, datasets=None,
                 daemon_addr='localhost', daemon_port=5300,
                 daemon_bin='./rbldnsd',
                 stderr=None):
        self._daemon = None
        self.datasets = []
        self.daemon_addr = daemon_addr
        self.daemon_port = daemon_port
        self.daemon_bin = daemon_bin
        self.stderr = stderr

    def add_dataset(self, ds_type, file, soa='example.com'):
        self.datasets.append((soa, ds_type, file))

    def __enter__(self):
        self._start_daemon()
        return self

    def __exit__(self, exc_type, exc_value, exc_tb):
        self._stop_daemon()

    def __del__(self):
        if self._daemon:
            self._stop_daemon()

    def query(self, name, qtype='TXT'):
        if not self._daemon:
            raise DaemonError("daemon not running")
        elif self._daemon.poll() is not None:
            raise DaemonError("daemon has died with code %d"
                              % self._daemon.returncode)

        req = DNS.Request(name=name, qtype=qtype, rd=0)
        resp = req.req(server=self.daemon_addr, port=self.daemon_port)
        status = resp.header['status']
        if status == 'REFUSED':
            raise QueryRefused("REFUSED")
        elif status == 'NXDOMAIN':
            return None
        else:
            assert status == 'NOERROR'
            assert len(resp.answers) == 1
            assert len(resp.answers[0]['data']) == 1
            return resp.answers[0]['data'][0]

    def _start_daemon(self):
        if len(self.datasets) == 0:
            raise ValueError("no datasets defined")

        cmd = [ self.daemon_bin, '-n',
                '-b', '%s/%u' % (self.daemon_addr, self.daemon_port),
                ]
        for zone, ds_type, file in self.datasets:
            if isinstance(file, basestring):
                filename = file
            else:
                filename = file.name
            cmd.append("%s:%s:%s" % (zone, ds_type, filename))

        self._stdout = TemporaryFile()
        self._daemon = daemon = subprocess.Popen(cmd, stdout=self._stdout,
                                                 stderr=self.stderr)

        # wait for rbldnsd to start responding
        time.sleep(0.1)
        for retry in count():
            if daemon.poll() is not None:
                raise DaemonError(
                    "rbldsnd exited unexpectedly with return code %d"
                    % daemon.returncode)
            try:
                self.query('dummy.nonexisting.zone')
                break
            except QueryRefused:
                break
            except DNS.DNSError as ex:
                if str(ex) != 'no working nameservers found':
                    raise
                elif retries > 10:
                    raise DaemonError(
                        "rbldnsd does not seem to be responding")

    def _stop_daemon(self):
        daemon = self._daemon

        if daemon.poll() is None:
            daemon.terminate()
            retries = count()
            while daemon.poll() is None:
                retry = next(retries)
                if retry == 30:
                    daemon.kill()
                elif retry == 50:
                    raise DaemonError("can not kill stop rbldnsd")
                time.sleep(0.1)

        self._daemon = None
        if daemon.returncode != 0:
            raise DaemonError("rbldnsd exited with code %d"
                              % daemon.returncode)

class TestRbldnsd(unittest.TestCase):
    def test(self):
        rbldnsd = Rbldnsd()
        test_zone = ZoneFile(lines=["1.2.3.4 :1: Success"])
        rbldnsd.add_dataset('ip4set', test_zone)
        with rbldnsd:
            self.assertEqual(rbldnsd.query('4.3.2.1.example.com'), 'Success')
            self.assertEqual(rbldnsd.query('5.3.2.1.example.com'), None)

if __name__ == '__main__':
    unittest.main()



""" Convenience script to run all python test cases in one go.
"""
import sys
import unittest

try:
    import DNS
except ImportError:
    print "TESTS SKIPPED: the pydns library is not installed"
    sys.exit(0)

from test_btrie import *
from test_ip6trie import *
from test_ip4trie import *
from test_acl import *

if __name__ == '__main__':
    unittest.main()
